pub enum ResolvedBoundVar<'a> {
Definite {
final_bound_var: u32,
ty: Defined<'a>,
},
Resource {
rtidx: u32,
},
}Expand description
Unlike [tv::ResolvedTyvar], which is mostly concerned with free
variables and leaves bound variables alone, this tells us the most
information that we have at codegen time for a top level bound
variable.
Variants
Definite
Fields
Resource