pub fn find_colliding_import_names(
imports: &[ExternDecl<'_>],
) -> HashSet<String>Expand description
Scan a list of import extern decls for interface name collisions. Returns short import names that need disambiguating.
pub fn find_colliding_import_names(
imports: &[ExternDecl<'_>],
) -> HashSet<String>Scan a list of import extern decls for interface name collisions. Returns short import names that need disambiguating.