pub enum FreeTyvar {
U(u32, u32),
E(u32, u32),
}Expand description
To make certain substitutions easier, free type variables are
divided into Universal and Existential variables. Each is
represented by a pair of indices: the first index is an index into
Ctx::parents(), specifying parent context has the variable
definition in it, and the second is an index into that context’s
Ctx::uvars or Ctx::evars.
Variants