pub enum Handleable {
Var(Tyvar),
Resource(ResourceId),
}Expand description
In the spec, this does not exist, but a validation rule ensures an invariant that certain deftype_e s are of this form.
Variants
Var(Tyvar)
Resource(ResourceId)