pub fn emit_value(s: &mut State<'_, '_>, vt: &Value<'_>) -> TokenStreamExpand description
Emit a Rust type corresponding to a given value type. The resultant token stream will parse as a Rust type.
Precondition: vt is an inline-able value type.
pub fn emit_value(s: &mut State<'_, '_>, vt: &Value<'_>) -> TokenStreamEmit a Rust type corresponding to a given value type. The resultant token stream will parse as a Rust type.
Precondition: vt is an inline-able value type.