pub enum ClosingError {
UnknownVar(bool, u32),
UnimportedVar(u32),
}Expand description
Closing can fail for a few reasons:
Variants
UnknownVar(bool, u32)
A variable was encountered that isn’t currently being moved to a bound variable. This is an internal invariant violation in the typechecker, not an issue of a malformed input type.
UnimportedVar(u32)
A universal variable wasn’t imported. This is probably an internal invariant violation in the typechecker.