pub struct Closing { /* private fields */ }Expand description
A closing substitution is used to map free variables into bound variables when converting a type being built in a context to a closed(ish) type that is above that context.
Like Opening, a given Closing substitution either affects
only existential variables or affects only universal variables, as
these are closed at different times.
Implementations