pub struct Opening { /* private fields */ }Expand description
An opening substitution is used to map bound variables into
free variables. Note that because of the differences in ordering
for bound variable indices (inside out) and context variables
(left to right, but variables are inserted in outside-in order),
Bound(0) gets mapped to Free(0, base + n).
Implementations