Module util

Module util 

General utilities for bindgen macros

Enums

WitSource
Input accepted by component bindgen.

Functions

emit_decls
Deal with $HYPERLIGHT_COMPONENT_MACRO_DEBUG: if it is present, save the given token stream (representing the result of macroexpansion) to the debug file and then return the token stream
read_wit_type
Read and parse a WIT type from a supported bindgen input.
read_wit_type_from_file
Read and parse a wasm-encoded WIT file, relative to the cargo manifest directory.