emit_decls

Function emit_decls 

pub fn emit_decls(decls: TokenStream) -> TokenStream
Deal with $HYPERLIGHT_COMPONENT_MACRO_DEBUG: if it is present, save the given token stream (representing the result of macroexpansion) to the debug file and then return the token stream