pub enum Error<'a> {
BareResourceExport,
BareComplexValTypeExport(Value<'a>),
DuplicateRecordField(Name<'a>),
DuplicateVariantField(Name<'a>),
DuplicateFlagsName(Name<'a>),
DuplicateEnumName(Name<'a>),
DuplicateExternName(&'a str, bool),
NotAResource(Error<'a>),
BorrowOutsideParam,
}Expand description
There are several ways in which a type may be ill-formed:
Variants
BareResourceExport
A component/instance exported a bare resource type not behind a tyvar (and therefore not named)
BareComplexValTypeExport(Value<'a>)
A component/instance exported certain complex value types not behind a tyvar (and therefore not named)
DuplicateRecordField(Name<'a>)
A record has multiple fields with the same name
DuplicateVariantField(Name<'a>)
A variant has multiple cases with the same name
DuplicateFlagsName(Name<'a>)
A flags has multiple flags with the same name
DuplicateEnumName(Name<'a>)
An enum has multiple cases with the same name
DuplicateExternName(&'a str, bool)
An import/export has the same name as another; the boolean is true if it is an import
NotAResource(Error<'a>)
A value type owns or borrows a type that is not a resource type
BorrowOutsideParam
A borrow type exists somewhere other than a function parameter