General utilities for bindgen macros
Enums
- WitSource
- Input accepted by component bindgen.
Functions
- emit_
decls - Deal with
$HYPERLIGHT_COMPONENT_MACRO_DEBUG: if it is present, save the given token stream (representing the result of macroexpansion) to the debug file and then return the token stream - read_
wit_ type - Read and parse a WIT type from a supported bindgen input.
- read_
wit_ type_ from_ file - Read and parse a wasm-encoded WIT file, relative to the cargo manifest directory.