General utilities for bindgen macros
Functions
- emit_
decls - Deal with
$HYPERLIGHT_COMPONENT_MACRO_DEBUG: if it is present, save the given token stream (representing the result of macroexpansion) to the debug file and then return the token stream - read_
wit_ type_ from_ file - Read and parse a WIT type encapsulated in a wasm file from the given filename, relative to the cargo manifest directory.